@@ -4,7 +4,8 @@
 #' `tally()` is a convenient wrapper for summarise that will either call
 #' [n()] or \code{\link{sum}(n)} depending on whether you're tallying
 #' for the first time, or re-tallying. `count()` is similar but calls
-#' [group_by()] before and [ungroup()] after.
+#' [group_by()] before and [ungroup()] after. If the data is already
+#' grouped, `count()` adds an additional group that is removed afterwards.
 #'
 #' `add_tally()` adds a column `n` to a table based on the number
 #' of items within each existing group, while `add_count()` is a shortcut that
@@ -13,16 +14,11 @@
 #' they add an additional column rather than collapsing each group.
 #'
 #' @note
-#' The column name in the returned data is usually `n`, even if you
-#' have supplied a weight.
+#' The column name in the returned data is given by the `name` argument,
+#' set to `"n"` by default.
 #'
-#' If the data already already has a column named `n`, the output column
-#' will be called `nn`. If the table already has columns called `n` and `nn`
-#' then the column returned will be `nnn`, and so on.
-#'
-#' There is currently no way to control the output variable name - if you
-#' need to change the default, you'll have to write the [summarise()]
-#' yourself.
+#' If the data already has a column by that name, the output column
+#' will be prefixed by an extra `"n"` as many times as necessary.
 #'
 #' @param x a [tbl()] to tally/count.
 #' @param ... Variables to group by.
@@ -36,34 +32,50 @@
 #'   frame. It supports [unquoting][rlang::quasiquotation]. See
 #'   `vignette("programming")` for an introduction to these concepts.
 #' @param sort if `TRUE` will sort output in descending order of `n`
+#' @param name The output column name. If omitted, it will be `n`.
+#' @param .drop see [group_by()]
 #' @return A tbl, grouped the same way as `x`.
 #' @export
 #' @examples
 #' # tally() is short-hand for summarise()
 #' mtcars %>% tally()
+#' mtcars %>% group_by(cyl) %>% tally()
 #' # count() is a short-hand for group_by() + tally()
 #' mtcars %>% count(cyl)
+#' # Note that if the data is already grouped, count() adds
+#' # an additional group that is removed afterwards
+#' mtcars %>% group_by(gear) %>% count(carb)
 #'
 #' # add_tally() is short-hand for mutate()
 #' mtcars %>% add_tally()
 #' # add_count() is a short-hand for group_by() + add_tally()
 #' mtcars %>% add_count(cyl)
 #'
-#' # count and tally are designed so that you can call
+#' # count() and tally() are designed so that you can call
 #' # them repeatedly, each time rolling up a level of detail
-#' species <- starwars %>% count(species, homeworld, sort = TRUE)
+#' species <-
+#'  starwars %>%
+#'  count(species, homeworld, sort = TRUE)
 #' species
 #' species %>% count(species, sort = TRUE)
 #'
+#' # Change the name of the newly created column:
+#' species <-
+#'  starwars %>%
+#'  count(species, homeworld, sort = TRUE, name = "n_species_by_homeworld")
+#' species
+#' species %>%
+#'  count(species, sort = TRUE, name = "n_species")
+#'
 #' # add_count() is useful for groupwise filtering
-#' # e.g.: show only species that have a single member
+#' # e.g.: show details for species that have a single member
 #' starwars %>%
 #'   add_count(species) %>%
 #'   filter(n == 1)
-tally <- function(x, wt, sort = FALSE) {
+tally <- function(x, wt = NULL, sort = FALSE, name = "n") {
   wt <- enquo(wt)
 
-  if (quo_is_missing(wt) && "n" %in% names(x)) {
+  if (quo_is_missing(wt) && "n" %in% tbl_vars(x)) {
     inform("Using `n` as weighting variable")
     wt <- quo(n)
   }
@@ -74,7 +86,12 @@ tally <- function(x, wt, sort = FALSE) {
     n <- quo(sum(!!wt, na.rm = TRUE))
   }
 
-  n_name <- n_name(tbl_vars(x))
+  n_name <- n_name(group_vars(x), name)
+
+  if (name != "n" && name %in% group_vars(x)) {
+    abort(glue("Column `{name}` already exists in grouped variables"))
+  }
+
   out <- summarise(x, !!n_name := !!n)
 
   if (sort) {
@@ -87,14 +104,21 @@ tally <- function(x, wt, sort = FALSE) {
 #' @inheritParams tally
 #' @export
 tally_ <- function(x, wt, sort = FALSE) {
+  signal_soft_deprecated(paste_line(
+    "tally_() is deprecated. ",
+    "Please use tally() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with tally() : https://tidyeval.tidyverse.org"
+  ))
+
   wt <- compat_lazy(wt, caller_env())
   tally(x, wt = !!wt, sort = sort)
 }
 
-n_name <- function(x) {
-  name <- "n"
+n_name <- function(x, name = "n") {
   while (name %in% x) {
-    name <- paste0(name, "n")
+    name <- paste0("n", name)
   }
 
   name
@@ -102,29 +126,39 @@ n_name <- function(x) {
 
 #' @export
 #' @rdname tally
-count <- function(x, ..., wt = NULL, sort = FALSE) {
+count <- function(x, ..., wt = NULL, sort = FALSE, name = "n", .drop = group_by_drop_default(x)) {
   groups <- group_vars(x)
 
-  x <- group_by(x, ..., add = TRUE)
-  x <- tally(x, wt = !!enquo(wt), sort = sort)
-  x <- group_by(x, !!!syms(groups), add = FALSE)
+  if (dots_n(...)) {
+    x <- .group_by_static_drop(x, ..., add = TRUE, .drop = .drop)
+  }
+  x <- tally(x, wt = !!enquo(wt), sort = sort, name = name)
+  x <- .group_by_static_drop(x, !!!syms(groups), add = FALSE, .drop = .drop)
   x
 }
 #' @export
 #' @rdname se-deprecated
-count_ <- function(x, vars, wt = NULL, sort = FALSE) {
+count_ <- function(x, vars, wt = NULL, sort = FALSE, .drop = group_by_drop_default(x)) {
+  signal_soft_deprecated(paste_line(
+    "count_() is deprecated. ",
+    "Please use count() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with count() : https://tidyeval.tidyverse.org"
+  ))
+
   vars <- compat_lazy_dots(vars, caller_env())
   wt <- wt %||% quo(NULL)
   wt <- compat_lazy(wt, caller_env())
-  count(x, !!!vars, wt = !!wt, sort = sort)
+  count(x, !!!vars, wt = !!wt, sort = sort, .drop = .drop)
 }
 
 #' @rdname tally
 #' @export
-add_tally <- function(x, wt, sort = FALSE) {
+add_tally <- function(x, wt, sort = FALSE, name = "n") {
   wt <- enquo(wt)
 
-  if (quo_is_missing(wt) && "n" %in% names(x)) {
+  if (quo_is_missing(wt) && "n" %in% tbl_vars(x)) {
     inform("Using `n` as weighting variable")
     wt <- quo(n)
   }
@@ -135,35 +169,54 @@ add_tally <- function(x, wt, sort = FALSE) {
     n <- quo(sum(!!wt, na.rm = TRUE))
   }
 
-  n_name <- n_name(tbl_vars(x))
+  n_name <- n_name(group_vars(x), name)
+
+  if (name != "n" && name %in% group_vars(x)) {
+    abort(glue("Column `{name}` already exists in grouped variables"))
+  }
+
   out <- mutate(x, !!n_name := !!n)
 
   if (sort) {
     out <- arrange(out, desc(!!sym(n_name)))
   }
 
-  grouped_df(out, group_vars(x))
+  grouped_df(out, group_vars(x), drop = group_by_drop_default(x))
 }
 #' @rdname se-deprecated
 #' @export
 add_tally_ <- function(x, wt, sort = FALSE) {
+  signal_soft_deprecated(paste_line(
+    "add_tally_() is deprecated. ",
+    "Please use add_tally() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with add_tally() : https://tidyeval.tidyverse.org"
+  ))
+
   wt <- compat_lazy(wt, caller_env())
   add_tally(x, !!wt, sort = sort)
 }
-
-
 #' @rdname tally
 #' @export
-add_count <- function(x, ..., wt = NULL, sort = FALSE) {
+add_count <- function(x, ..., wt = NULL, sort = FALSE, name = "n") {
   g <- group_vars(x)
   grouped <- group_by(x, ..., add = TRUE)
 
-  out <- add_tally(grouped, wt = !!enquo(wt), sort = sort)
-  grouped_df(out, g)
+  out <- add_tally(grouped, wt = !!enquo(wt), sort = sort, name = name)
+  grouped_df(out, g, drop = group_by_drop_default(grouped))
 }
 #' @rdname se-deprecated
 #' @export
 add_count_ <- function(x, vars, wt = NULL, sort = FALSE) {
+  signal_soft_deprecated(paste_line(
+    "add_count_() is deprecated. ",
+    "Please use add_count() instead",
+    "",
+    "The 'programming' vignette or the tidyeval book can help you",
+    "to program with add_count() : https://tidyeval.tidyverse.org"
+  ))
+
   vars <- compat_lazy_dots(vars, caller_env())
   wt <- wt %||% quo(NULL)
   wt <- compat_lazy(wt, caller_env())
