@@ -2,68 +2,81 @@
 #include <dplyr/main.h>
 
 #include <tools/utils.h>
-#include <dplyr/white_list.h>
+#include <dplyr/allow_list.h>
 #include <tools/collapse.h>
-#include <dplyr/bad.h>
+#include <tools/bad.h>
+#include <dplyr/data/GroupedDataFrame.h>
+#include <dplyr/symbols.h>
+#include <dplyr/lifecycle.h>
 
-using namespace Rcpp;
+SEXP child_env(SEXP parent) {
+  Rcpp::Shield<SEXP> call(Rf_lang3(dplyr::symbols::new_env, Rf_ScalarLogical(TRUE), parent));
+  return Rf_eval(call, R_BaseEnv);
+}
 
-// [[Rcpp::export]]
-void check_valid_names(const CharacterVector& names, bool warn_only = false) {
-  IntegerVector which_na;
-  for (int i = 0; i < names.size(); ++i) {
-    if (String(names[i]) == R_NaString) {
+// [[Rcpp::export(rng = false)]]
+void check_valid_names(const Rcpp::CharacterVector& names, bool warn_only = false) {
+  R_xlen_t n = XLENGTH(names);
+
+  std::vector<int> which_na;
+  which_na.reserve(n);
+
+  for (int i = 0; i < n; ++i) {
+    if (STRING_ELT(names, i) == R_NaString) {
       which_na.push_back(i + 1);
     }
   }
 
   if (which_na.size() > 0) {
-    String msg = msg_bad_cols(SymbolVector(static_cast<SEXP>(which_na)), "cannot have NA as name");
+    dplyr::SymbolVector which_na_symbols(Rcpp::wrap(which_na));
+    Rcpp::String msg = msg_bad_cols(which_na_symbols, "cannot have NA as name");
     if (warn_only)
-      warning(msg.get_cstring());
+      Rcpp::warning(msg.get_cstring());
     else
-      stop(msg.get_cstring());
+      Rcpp::stop(msg.get_cstring());
   }
 
-  LogicalVector dup = duplicated(names);
+  Rcpp::LogicalVector dup(duplicated(names));
   if (any(dup).is_true()) {
-    String msg = msg_bad_cols(SymbolVector(static_cast<SEXP>(names[dup])), "must have a unique name");
+    Rcpp::String msg = msg_bad_cols(dplyr::SymbolVector(static_cast<SEXP>(names[dup])), "must have a unique name");
     if (warn_only)
-      warning(msg.get_cstring());
+      Rcpp::warning(msg.get_cstring());
     else
-      stop(msg.get_cstring());
+      Rcpp::stop(msg.get_cstring());
   }
 }
 
 // Need forwarder to avoid compilation warning for default argument
-void check_valid_colnames(const DataFrame& df, bool warn_only) {
-  check_valid_names(vec_names_or_empty(df), warn_only);
+void check_valid_colnames(const Rcpp::DataFrame& df, bool warn_only) {
+  Rcpp::Shield<SEXP> names(vec_names_or_empty(df));
+  check_valid_names((SEXP)names, warn_only);
 }
 
-void check_range_one_based(int x, int max) {
+int check_range_one_based(int x, int max) {
   // Also covers NA
   if (x <= 0 || x > max) {
-    stop("Index out of range");
+    Rcpp::stop("Index out of range");
   }
+  return x;
 }
 
-// [[Rcpp::export]]
-void assert_all_white_list(const DataFrame& data) {
-  // checking variables are on the white list
+// [[Rcpp::export(rng = false)]]
+void assert_all_allow_list(const Rcpp::DataFrame& data) {
+  // checking variables are on the allow list
   int nc = data.size();
   for (int i = 0; i < nc; i++) {
-    if (!white_list(data[i])) {
-      SymbolVector names = data.names();
-      const SymbolString& name_i = names[i];
+    if (!dplyr::allow_list(data[i])) {
+      dplyr::SymbolVector names(Rf_getAttrib(data, dplyr::symbols::names));
+      const dplyr::SymbolString& name_i = names[i];
       SEXP v = data[i];
 
       SEXP klass = Rf_getAttrib(v, R_ClassSymbol);
       if (!Rf_isNull(klass)) {
         bad_col(name_i, "is of unsupported class {type}",
-                _["type"] = get_single_class(v));
+                Rcpp::_["type"] = dplyr::get_single_class(v));
       }
       else {
-        bad_col(name_i, "is of unsupported type {type}", _["type"] = Rf_type2char(TYPEOF(v)));
+        bad_col(name_i, "is of unsupported type {type}", Rcpp::_["type"] = Rf_type2char(TYPEOF(v)));
       }
     }
   }
@@ -74,10 +87,9 @@ SEXP shared_SEXP(SEXP x) {
   return x;
 }
 
-// [[Rcpp::export]]
-SEXP shallow_copy(const List& data) {
+SEXP shallow_copy(const Rcpp::List& data) {
   int n = data.size();
-  List out(n);
+  Rcpp::List out(n);
   for (int i = 0; i < n; i++) {
     out[i] = shared_SEXP(data[i]);
   }
@@ -86,12 +98,12 @@ SEXP shallow_copy(const List& data) {
 }
 
 SEXP pairlist_shallow_copy(SEXP p) {
-  Shield<SEXP> attr(Rf_cons(CAR(p), R_NilValue));
+  Rcpp::Shield<SEXP> attr(Rf_cons(CAR(p), R_NilValue));
   SEXP q = attr;
   SET_TAG(q, TAG(p));
   p = CDR(p);
   while (!Rf_isNull(p)) {
-    Shield<SEXP> s(Rf_cons(CAR(p), R_NilValue));
+    Rcpp::Shield<SEXP> s(Rf_cons(CAR(p), R_NilValue));
     SETCDR(q, s);
     q = CDR(q);
     SET_TAG(q, TAG(p));
@@ -104,7 +116,7 @@ void copy_only_attributes(SEXP out, SEXP data) {
   SEXP att = ATTRIB(data);
   const bool has_attributes = !Rf_isNull(att);
   if (has_attributes) {
-    LOG_VERBOSE << "copying attributes: " << CharacterVector(List(att).names());
+    LOG_VERBOSE << "copying attributes: " << Rcpp::CharacterVector(Rf_getAttrib(Rcpp::List(att), dplyr::symbols::names));
 
     SET_ATTRIB(out, pairlist_shallow_copy(ATTRIB(data)));
   }
@@ -129,7 +141,7 @@ namespace dplyr {
 std::string get_single_class(SEXP x) {
   SEXP klass = Rf_getAttrib(x, R_ClassSymbol);
   if (!Rf_isNull(klass)) {
-    CharacterVector classes(klass);
+    Rcpp::CharacterVector classes(klass);
     return collapse_utf8(classes, "/");
   }
 
@@ -138,6 +150,8 @@ std::string get_single_class(SEXP x) {
   }
 
   switch (TYPEOF(x)) {
+  case RAWSXP:
+    return "raw";
   case INTSXP:
     return "integer";
   case REALSXP :
@@ -146,6 +160,8 @@ std::string get_single_class(SEXP x) {
     return "logical";
   case STRSXP:
     return "character";
+  case CPLXSXP:
+    return "complex";
 
   case VECSXP:
     return "list";
@@ -154,33 +170,44 @@ std::string get_single_class(SEXP x) {
   }
 
   // just call R to deal with other cases
-  // we could call R_data_class directly but we might get a "this is not part of the api"
-  RObject class_call(Rf_lang2(Rf_install("class"), x));
+  Rcpp::RObject class_call(Rf_lang2(R_ClassSymbol, x));
   klass = Rf_eval(class_call, R_GlobalEnv);
   return CHAR(STRING_ELT(klass, 0));
 }
 
-CharacterVector default_chars(SEXP x, R_xlen_t len) {
-  if (Rf_isNull(x)) return CharacterVector(len);
+Rcpp::CharacterVector default_chars(SEXP x, R_xlen_t len) {
+  if (Rf_isNull(x)) return Rcpp::CharacterVector(len);
   return x;
 }
 
-CharacterVector get_class(SEXP x) {
+Rcpp::CharacterVector get_class(SEXP x) {
   SEXP class_attr = Rf_getAttrib(x, R_ClassSymbol);
   return default_chars(class_attr, 0);
 }
 
-SEXP set_class(SEXP x, const CharacterVector& class_) {
+void copy_attrib(SEXP out, SEXP origin, SEXP symbol) {
+  Rf_setAttrib(out, symbol, Rcpp::Shield<SEXP>(Rf_getAttrib(origin, symbol)));
+}
+
+void copy_class(SEXP out, SEXP origin) {
+  copy_attrib(out, origin, R_ClassSymbol);
+}
+
+void copy_names(SEXP out, SEXP origin) {
+  copy_attrib(out, origin, R_NamesSymbol);
+}
+
+SEXP set_class(SEXP x, const Rcpp::CharacterVector& class_) {
   SEXP class_attr = class_.length() == 0 ? R_NilValue : (SEXP)class_;
   return Rf_setAttrib(x, R_ClassSymbol, class_attr);
 }
 
-CharacterVector get_levels(SEXP x) {
+Rcpp::CharacterVector get_levels(SEXP x) {
   SEXP levels_attr = Rf_getAttrib(x, R_LevelsSymbol);
   return default_chars(levels_attr, 0);
 }
 
-SEXP set_levels(SEXP x, const CharacterVector& levels) {
+SEXP set_levels(SEXP x, const Rcpp::CharacterVector& levels) {
   return Rf_setAttrib(x, R_LevelsSymbol, levels);
 }
 
@@ -190,7 +217,7 @@ bool same_levels(SEXP left, SEXP right) {
 
 SEXP list_as_chr(SEXP x) {
   int n = Rf_length(x);
-  CharacterVector chr(n);
+  Rcpp::CharacterVector chr(n);
 
   for (int i = 0; i != n; ++i) {
     SEXP elt = VECTOR_ELT(x, i);
@@ -208,41 +235,13 @@ SEXP list_as_chr(SEXP x) {
       break;
     }
 
-    stop("The tibble's `vars` attribute has unexpected contents");
+    Rcpp::stop("corrupt grouped data frame");
   }
 
   return chr;
 }
 
-SymbolVector get_vars(SEXP x, bool duplicate) {
-  static SEXP vars_symbol = Rf_install("vars");
-  RObject vars = Rf_getAttrib(x, vars_symbol);
-  if (duplicate && MAYBE_SHARED(vars)) vars = Rf_duplicate(vars);
-
-  switch (TYPEOF(vars)) {
-  case NILSXP:
-  case STRSXP:
-    break;
-  case VECSXP:
-    vars = list_as_chr(vars);
-    break;
-  default:
-    stop("The tibble's `vars` attribute has unexpected type");
-  }
-
-  return SymbolVector(vars);
-}
-
-void set_vars(SEXP x, const SymbolVector& vars) {
-  static SEXP vars_symbol = Rf_install("vars");
-  Rf_setAttrib(x, vars_symbol, null_if_empty(vars.get_vector()));
-}
-
-void copy_vars(SEXP target, SEXP source) {
-  set_vars(target, get_vars(source));
-}
-
-bool character_vector_equal(const CharacterVector& x, const CharacterVector& y) {
+bool character_vector_equal(const Rcpp::CharacterVector& x, const Rcpp::CharacterVector& y) {
   if ((SEXP)x == (SEXP)y) return true;
 
   if (x.length() != y.length())
@@ -317,28 +316,81 @@ bool has_name_at(SEXP x, R_len_t i) {
   return TYPEOF(nms) == STRSXP && !is_str_empty(STRING_ELT(nms, i));
 }
 
-SEXP name_at(SEXP x, size_t i) {
-  SEXP names = vec_names(x);
-  if (Rf_isNull(names))
-    return Rf_mkChar("");
-  else
-    return STRING_ELT(names, i);
+// [[Rcpp::export(rng = false)]]
+bool is_data_pronoun(SEXP expr) {
+  if (TYPEOF(expr) != LANGSXP || Rf_length(expr) != 3)
+    return false;
+
+  SEXP first = CADR(expr);
+  if (first != dplyr::symbols::dot_data)
+    return false;
+
+  SEXP second = CADDR(expr);
+  SEXP fun = CAR(expr);
+
+  // .data$x or .data$"x"
+  if (fun == R_DollarSymbol && (TYPEOF(second) == SYMSXP || TYPEOF(second) == STRSXP))
+    return true;
+
+  // .data[["x"]]
+  if (fun == R_Bracket2Symbol && TYPEOF(second) == STRSXP)
+    return true;
+
+  return false;
 }
 
-SEXP f_env(SEXP x) {
-  return Rf_getAttrib(x, Rf_install(".Environment"));
+// [[Rcpp::export(rng = false)]]
+bool is_variable_reference(SEXP expr) {
+  // x
+  if (TYPEOF(expr) == SYMSXP)
+    return true;
+
+  return is_data_pronoun(expr);
 }
 
-bool is_quosure(SEXP x) {
-  return TYPEOF(x) == LANGSXP
-         && Rf_length(x) == 2
-         && Rf_inherits(x, "quosure")
-         && TYPEOF(f_env(x)) == ENVSXP;
+// [[Rcpp::export(rng = false)]]
+bool quo_is_variable_reference(SEXP quo) {
+  return is_variable_reference(CADR(quo));
 }
 
-SEXP maybe_rhs(SEXP x) {
-  if (is_quosure(x))
-    return CADR(x);
-  else
-    return x;
+// [[Rcpp::export(rng = false)]]
+bool quo_is_data_pronoun(SEXP quo) {
+  return is_data_pronoun(CADR(quo));
 }
+
+int get_size(SEXP x) {
+  if (Rf_isMatrix(x)) {
+    return INTEGER(Rf_getAttrib(x, R_DimSymbol))[0];
+  } else if (Rf_inherits(x, "data.frame")) {
+    return Rcpp::DataFrame(x).nrows();
+  } else {
+    return Rf_length(x);
+  }
+}
+
+namespace dplyr {
+namespace lifecycle {
+
+void warn_deprecated(const std::string& s) {
+  static Rcpp::Environment ns_dplyr(Rcpp::Environment::namespace_env("dplyr"));
+
+  Rcpp::CharacterVector msg(Rcpp::CharacterVector::create(s));
+  Rcpp::Shield<SEXP> call(Rf_lang2(symbols::warn_deprecated, msg));
+
+  Rcpp::Rcpp_eval(call, ns_dplyr);
+}
+
+void signal_soft_deprecated(const std::string& s, SEXP caller_env) {
+  static Rcpp::Environment ns_dplyr(Rcpp::Environment::namespace_env("dplyr"));
+
+  Rcpp::CharacterVector msg(Rcpp::CharacterVector::create(s));
+  Rcpp::Shield<SEXP> call(Rf_lang4(symbols::signal_soft_deprecated, msg, msg, caller_env));
+
+  Rcpp::Rcpp_eval(call, ns_dplyr);
+}
+
+
+}
+}
+
+
