@@ -106,9 +106,9 @@ test_that("mutate recycles results of length 1", {
   list <- list(NULL)
 
   res <- mutate(group_by(df, x), int = int, str = str, num = num, bool = bool, list = list)
-  expect_equal(res$int , rep(int , 4))
-  expect_equal(res$str , rep(str , 4))
-  expect_equal(res$num , rep(num , 4))
+  expect_equal(res$int, rep(int, 4))
+  expect_equal(res$str, rep(str, 4))
+  expect_equal(res$num, rep(num, 4))
   expect_equal(res$bool, rep(bool, 4))
   expect_equal(res$list, rep(list, 4))
 })
@@ -117,7 +117,7 @@ test_that("mutate recycles results of length 1", {
 test_that("mutate handles out of data variables", {
   today <- Sys.Date()
   now <- Sys.time()
-  df  <- data.frame(x = c(2, 2, 3, 3))
+  df <- data.frame(x = c(2, 2, 3, 3))
   gdf <- group_by(df, x)
 
   int  <- c(1L, 2L)
@@ -131,12 +131,12 @@ test_that("mutate handles out of data variables", {
     gdf,
     int = int, str = str, num = num, bool = bool, dat = dat, tim = tim
   )
-  expect_equal(res$int , rep(int , 2))
-  expect_equal(res$str , rep(str , 2))
-  expect_equal(res$num , rep(num , 2))
+  expect_equal(res$int, rep(int, 2))
+  expect_equal(res$str, rep(str, 2))
+  expect_equal(res$num, rep(num, 2))
   expect_equal(res$bool, rep(bool, 2))
-  expect_equal(res$dat , rep(dat , 2))
-  expect_equal(res$tim , rep(tim , 2))
+  expect_equal(res$dat, rep(dat, 2))
+  expect_equal(res$tim, rep(tim, 2))
 
   int <- 1:6
   expect_error(
@@ -161,12 +161,12 @@ test_that("mutate handles out of data variables", {
     tbl_df(df),
     int = int, str = str, num = num, bool = bool, tim = tim, dat = dat
   )
-  expect_equal(res$int , int)
-  expect_equal(res$str , str)
-  expect_equal(res$num , num)
+  expect_equal(res$int, int)
+  expect_equal(res$str, str)
+  expect_equal(res$num, num)
   expect_equal(res$bool, bool)
-  expect_equal(res$dat , dat)
-  expect_equal(res$tim , tim)
+  expect_equal(res$dat, dat)
+  expect_equal(res$tim, tim)
 })
 
 test_that("mutate handles passing ...", {
@@ -204,14 +204,13 @@ test_that("mutate handles passing ...", {
   expect_equal(res$x2, rep(2, 4))
   expect_equal(res$before, rep("before", 4))
   expect_equal(res$after, rep("after", 4))
-
 })
 
 test_that("mutate fails on unsupported column type", {
   df <- data.frame(created = c("2014/1/1", "2014/1/2", "2014/1/2"))
   expect_error(
     mutate(df, date = strptime(created, "%Y/%m/%d")),
-    "Column `date` is of unsupported class POSIXlt",
+    "Column `date` is of unsupported class POSIXlt; please use POSIXct instead",
     fixed = TRUE
   )
 
@@ -221,11 +220,21 @@ test_that("mutate fails on unsupported column type", {
   )
   expect_error(
     mutate(group_by(df, g), date = strptime(created, "%Y/%m/%d")),
-    "Column `date` is of unsupported class POSIXlt",
+    "Column `date` is of unsupported class POSIXlt; please use POSIXct instead",
     fixed = TRUE
   )
 })
 
+test_that("mutate can handle POSIXlt columns (#3854)", {
+  df <- data.frame(g=c(1,1,3))
+  df$created <- strptime(c("2014/1/1", "2014/1/2", "2014/1/2"), format = "%Y/%m/%d")
+
+  res <- df %>%
+    group_by(g) %>%
+    mutate(Y = format(created, "%Y"))
+  expect_true(all(res$Y == "2014"))
+})
+
 test_that("mutate modifies same column repeatedly (#243)", {
   df <- data.frame(x = 1)
   expect_equal(mutate(df, x = x + 1, x = x + 1)$x, 3)
@@ -241,10 +250,15 @@ test_that("mutate errors when results are not compatible accross groups (#299)",
 })
 
 test_that("assignments don't overwrite variables (#315)", {
-   expect_equal(
-     mutate(mtcars, cyl2 = { mpg <- cyl ^ 2; -mpg }),
-     mutate(mtcars, cyl2 = -cyl ^ 2)
-   )
+  expect_equal(
+    mutate(mtcars,
+      cyl2 = {
+        mpg <- cyl^2
+        -mpg
+      }
+    ),
+    mutate(mtcars, cyl2 = -cyl^2)
+  )
 })
 
 test_that("hybrid evaluator uses correct environment (#403)", {
@@ -265,52 +279,55 @@ test_that("mutate remove variables with = NULL syntax (#462)", {
 })
 
 test_that("mutate strips names, but only if grouped (#1689, #2675)", {
-  data <- data_frame(a = 1:3) %>% mutate(b = setNames(nm = a))
+  data <- tibble(a = 1:3) %>% mutate(b = setNames(nm = a))
   expect_equal(names(data$b), as.character(1:3))
 
-  data <- data_frame(a = 1:3) %>% rowwise %>% mutate(b = setNames(nm = a))
+  data <- tibble(a = 1:3) %>% rowwise() %>% mutate(b = setNames(nm = a))
   expect_null(names(data$b))
 
-  data <- data_frame(a = c(1, 1, 2)) %>% group_by(a) %>% mutate(b = setNames(nm = a))
+  data <- tibble(a = c(1, 1, 2)) %>% group_by(a) %>% mutate(b = setNames(nm = a))
   expect_null(names(data$b))
 })
 
 test_that("mutate does not strip names of list-columns (#2675)", {
   vec <- list(a = 1, b = 2)
-  data <- data_frame(x = vec)
+  data <- tibble(x = vec)
   data <- mutate(data, x)
   expect_identical(names(vec), c("a", "b"))
   expect_identical(names(data$x), c("a", "b"))
 })
 
-test_that("mutate gives a nice error message if an expression evaluates to NULL (#2187)", {
-  df <- data_frame(a = 1:3)
+test_that("mutate removes columns when the expression evaluates to NULL for all groups (#2945)", {
+  df <- tibble(a = 1:3, b=4:6)
   gf <- group_by(df, a)
   rf <- rowwise(df)
 
-  expect_error(
+  expect_equal(
     mutate(df, b = identity(NULL)),
-    "Column `b` is of unsupported type NULL",
-    fixed = TRUE
+    select(df, -b)
   )
-  expect_error(
+  expect_equal(
     mutate(gf, b = identity(NULL)),
-    "Column `b` is of unsupported type NULL",
-    fixed = TRUE
+    select(gf, -b)
   )
-  expect_error(
+  expect_equal(
     mutate(rf, b = identity(NULL)),
-    "Column `b` is of unsupported type NULL",
-    fixed = TRUE
+    select(rf,-b)
   )
 })
 
+test_that("mutate treats NULL specially when the expression sometimes evaulates to NULL (#2945)", {
+  df <- tibble(a = 1:3, b=4:6) %>% group_by(a)
+  expect_equal( mutate(df, if(a==1) NULL else "foo") %>% pull(), c(NA, "foo", "foo"))
+  expect_equal( mutate(df, if(a==1) NULL else list(b)) %>% pull(), list(NULL, 5L, 6L))
+})
+
 test_that("mutate(rowwise_df) makes a rowwise_df (#463)", {
   one_mod <- data.frame(grp = "a", x = runif(5, 0, 1)) %>%
-    tbl_df %>%
+    tbl_df() %>%
     mutate(y = rnorm(x, x * 2, 1)) %>%
     group_by(grp) %>%
-    do(mod = lm(y~x, data = .))
+    do(mod = lm(y ~ x, data = .))
 
   out <- one_mod %>%
     mutate(rsq = summary(mod)$r.squared) %>%
@@ -323,7 +340,7 @@ test_that("mutate(rowwise_df) makes a rowwise_df (#463)", {
 })
 
 test_that("mutate allows list columns (#555)", {
-  df  <- data.frame(x = c("a;b", "c;d;e"), stringsAsFactors = FALSE)
+  df <- data.frame(x = c("a;b", "c;d;e"), stringsAsFactors = FALSE)
   res <- mutate(df, pieces = strsplit(x, ";"))
   expect_equal(res$pieces, list(c("a", "b"), c("c", "d", "e")))
 })
@@ -353,15 +370,15 @@ test_that("namespace extraction works in hybrid (#412)", {
 })
 
 test_that("hybrid not get in the way of order_by (#169)", {
-  df <- data_frame(x = 10:1, y = 1:10)
+  df <- tibble(x = 10:1, y = 1:10)
   res <- mutate(df, z = order_by(x, cumsum(y)))
   expect_equal(res$z, rev(cumsum(10:1)))
 })
 
 test_that("mutate supports difftime objects (#390)", {
-  df <- data_frame(
-    grp =   c(1, 1,  2, 2),
-    val =   c(1, 3,  4, 6),
+  df <- tibble(
+    grp = c(1, 1, 2, 2),
+    val = c(1, 3, 4, 6),
     date1 = c(rep(Sys.Date() - 10, 2), rep(Sys.Date() - 20, 2)),
     date2 = Sys.Date() + c(1, 2, 1, 2),
     diffdate = difftime(date2, date1, unit = "days")
@@ -380,23 +397,29 @@ test_that("mutate supports difftime objects (#390)", {
 
 test_that("mutate works on zero-row grouped data frame (#596)", {
   dat <- data.frame(a = numeric(0), b = character(0))
-  res <- dat %>% group_by(b) %>% mutate(a2 = a * 2)
+  res <- dat %>% group_by(b, .drop = FALSE) %>% mutate(a2 = a * 2)
   expect_is(res$a2, "numeric")
   expect_is(res, "grouped_df")
   expect_equal(res$a2, numeric(0))
-  expect_equal(attr(res, "indices"), list())
-  expect_equal(attr(res, "vars"), "b")
-  expect_equal(attr(res, "group_sizes"), integer(0))
-  expect_equal(attr(res, "biggest_group_size"), 0L)
+
+  expect_equal(group_rows(res), list())
+  expect_equal(group_data(res)$b, factor(character(0)))
+})
+
+test_that("mutate works on zero-row rowwise data frame (#4224)", {
+  dat <- data.frame(a = numeric(0))
+  res <- dat %>% rowwise() %>% mutate(a2 = a * 2)
+  expect_is(res$a2, "numeric")
+  expect_is(res, "rowwise_df")
+  expect_equal(res$a2, numeric(0))
 })
 
 test_that("Non-ascii column names in version 0.3 are not duplicated (#636)", {
-  # Currently failing (#2967)
-  skip_on_os("windows")
-  df <- data_frame(a = "1", b = "2")
+  skip("Currently failing (#2967)")
+  df <- tibble(a = "1", b = "2")
   names(df) <- c("a", enc2native("\u4e2d"))
 
-  res <- df %>% mutate_all(funs(as.numeric)) %>% names
+  res <- df %>% mutate_all(funs(as.numeric)) %>% names()
   expect_equal(res, names(df))
 })
 
@@ -406,7 +429,7 @@ test_that("nested hybrid functions do the right thing (#637)", {
 })
 
 test_that("mutate handles using and gathering complex data (#436)", {
-  d <- data_frame(x = 1:10, y = 1:10 + 2i)
+  d <- tibble(x = 1:10, y = 1:10 + 2i)
   res <- mutate(d, real = Re(y), imag = Im(y), z = 2 * y, constant = 2 + 2i)
   expect_equal(names(res), c("x", "y", "real", "imag", "z", "constant"))
   expect_equal(res$real, Re(d$y))
@@ -419,7 +442,7 @@ test_that("mutate forbids POSIXlt results (#670)", {
   expect_error(
     data.frame(time = "2014/01/01 10:10:10") %>%
       mutate(time = as.POSIXlt(time)),
-    "Column `time` is of unsupported class POSIXlt",
+    "Column `time` is of unsupported class POSIXlt; please use POSIXct instead",
     fixed = TRUE
   )
 
@@ -427,14 +450,13 @@ test_that("mutate forbids POSIXlt results (#670)", {
     data.frame(time = "2014/01/01 10:10:10", a = 2) %>%
       group_by(a) %>%
       mutate(time = as.POSIXlt(time)),
-    "Column `time` is of unsupported class POSIXlt",
+    "Column `time` is of unsupported class POSIXlt; please use POSIXct instead",
     fixed = TRUE
   )
-
 })
 
 test_that("constant factor can be handled by mutate (#715)", {
-  d <- data_frame(x = 1:2) %>% mutate(y = factor("A"))
+  d <- tibble(x = 1:2) %>% mutate(y = factor("A"))
   expect_true(is.factor(d$y))
   expect_equal(d$y, factor(c("A", "A")))
 })
@@ -452,20 +474,36 @@ test_that("row_number handles empty data frames (#762)", {
   )
   expect_equal(
     names(res),
-     c("a", "row_number_0", "row_number_a", "ntile", "min_rank", "percent_rank", "dense_rank", "cume_dist")
-   )
+    c("a", "row_number_0", "row_number_a", "ntile", "min_rank", "percent_rank", "dense_rank", "cume_dist")
+  )
   expect_equal(nrow(res), 0L)
 })
 
-test_that("no utf8 invasion (#722)", {
-  skip_on_os("windows")
+test_that("hybrid rank functions handle NA (#4427)", {
+  df <- tibble(a = runif(1000, -1, 1), b = runif(1000, -1, 1))
+  df[df < 0] <- NA
+  df <- df %>%
+    mutate(
+      gain = b - a,
+      cume_dist_hybrid = cume_dist(gain),
+      cume_dist_std = cume_dist(b - a),
+      pct_rank_hybrid = percent_rank(gain),
+      pct_rank_std = percent_rank(b-a)
+    )
 
-  source("utf-8.R", local = TRUE, encoding = "UTF-8")
+  expect_equal(df$cume_dist_hybrid, df$cume_dist_std)
+  expect_equal(df$pct_rank_hybrid, df$pct_rank_std)
+})
+
+test_that("no utf8 invasion (#722)", {
+  skip("fails on windows, but also on one cran machine")
+
+  source("utf-8.txt", local = TRUE, encoding = "UTF-8")
 })
 
 test_that("mutate works on empty data frames (#1142)", {
   df <- data.frame()
-  res <- df %>% mutate
+  res <- df %>% mutate()
   expect_equal(nrow(res), 0L)
   expect_equal(length(res), 0L)
 
@@ -476,8 +514,8 @@ test_that("mutate works on empty data frames (#1142)", {
 })
 
 test_that("mutate handles 0 rows rowwise (#1300)", {
-  a <- data_frame(x = 1)
-  b <- data_frame(y = character())
+  a <- tibble(x = 1)
+  b <- tibble(y = character())
 
   g <- function(y) {
     1
@@ -488,16 +526,23 @@ test_that("mutate handles 0 rows rowwise (#1300)", {
 
   res <- f()
   expect_equal(nrow(res), 0L)
+})
+
+test_that("rhs of mutate cannot be a data frame (#3298)", {
+  df <- data.frame("a" = c(1, 2, 3), "b" = c(2, 3, 4), "base_col" = c(3, 4, 5))
+  expect_error(
+    mutate(df, new_col = data.frame(1:3)),
+    "Column `new_col` is of unsupported class data.frame"
+  )
 
   expect_error(
-    a %>% mutate(b = f()),
-    "Column `b` must be length 1 (the number of rows), not 2",
-    fixed = TRUE
+    mutate(group_by(df, a), new_col = data.frame(1:3)),
+    "Column `new_col` is of unsupported class data.frame"
   )
+
   expect_error(
-    a %>% rowwise() %>% mutate(b = f()),
-    "Column `b` must be length 1 (the group size), not 2",
-    fixed = TRUE
+    mutate(rowwise(df), new_col = data.frame(1:3)),
+    "Column `new_col` is of unsupported class data.frame"
   )
 })
 
@@ -543,12 +588,12 @@ test_that("mutate handles the all NA case (#958)", {
 
 test_that("rowwise mutate gives expected results (#1381)", {
   f <- function(x) ifelse(x < 2, NA_real_, x)
-  res <- data_frame(x = 1:3) %>% rowwise() %>% mutate(y = f(x))
+  res <- tibble(x = 1:3) %>% rowwise() %>% mutate(y = f(x))
   expect_equal(res$y, c(NA, 2, 3))
 })
 
 test_that("mutate handles factors (#1414)", {
-  d <- data_frame(
+  d <- tibble(
     g = c(1, 1, 1, 2, 2, 3, 3),
     f = c("a", "b", "a", "a", "a", "b", "b")
   )
@@ -557,7 +602,7 @@ test_that("mutate handles factors (#1414)", {
 })
 
 test_that("mutate handles results from one group with all NA values (#1463) ", {
-  df <- data_frame(x = c(1, 2), y = c(1, NA))
+  df <- tibble(x = c(1, 2), y = c(1, NA))
   res <- df %>% group_by(x) %>% mutate(z = ifelse(y > 1, 1, 2))
   expect_true(is.na(res$z[2]))
   expect_is(res$z, "numeric")
@@ -586,11 +631,11 @@ test_that("mutate disambiguates NA and NaN (#1448)", {
   expect_true(is.nan(res$pass2[1]))
 
   res <- Pass %>%
-    rowwise %>%
+    rowwise() %>%
     mutate(pass2 = P2 / (P2 + F2))
   expect_true(is.nan(res$pass2[1]))
 
-  Pass <- data_frame(
+  Pass <- tibble(
     P1 = c(2L, 0L, 10L, 8L, 9L),
     F1 = c(0L, 2L, 0L, 4L, 3L),
     P2 = c(0L, 3L, 2L, 2L, 2L),
@@ -609,7 +654,7 @@ test_that("mutate disambiguates NA and NaN (#1448)", {
 })
 
 test_that("hybrid evaluator leaves formulas untouched (#1447)", {
-  d <- data_frame(g = 1:2, training = list(mtcars, mtcars * 2))
+  d <- tibble(g = 1:2, training = list(mtcars, mtcars * 2))
   mpg <- data.frame(x = 1:10, y = 1:10)
   res <- d %>%
     group_by(g) %>%
@@ -620,7 +665,7 @@ test_that("hybrid evaluator leaves formulas untouched (#1447)", {
 })
 
 test_that("lead/lag inside mutate handles expressions as value for default (#1411) ", {
-  df <- data_frame(x = 1:3)
+  df <- tibble(x = 1:3)
   res <- mutate(df, leadn = lead(x, default = x[1]), lagn = lag(x, default = x[1]))
   expect_equal(res$leadn, lead(df$x, default = df$x[1]))
   expect_equal(res$lagn, lag(df$x, default = df$x[1]))
@@ -638,7 +683,7 @@ test_that("grouped mutate does not drop grouping attributes (#1020)", {
 })
 
 test_that("grouped mutate coerces integer + double -> double (#1892)", {
-  df <- data_frame(
+  df <- tibble(
     id = c(1, 4),
     value = c(1L, NA),
     group = c("A", "B")
@@ -650,7 +695,7 @@ test_that("grouped mutate coerces integer + double -> double (#1892)", {
 })
 
 test_that("grouped mutate coerces factor + character -> character (WARN) (#1892)", {
-  factor_or_character <- function(x)  {
+  factor_or_character <- function(x) {
     if (x > 3) {
       return(factor("hello"))
     } else {
@@ -658,7 +703,7 @@ test_that("grouped mutate coerces factor + character -> character (WARN) (#1892)
     }
   }
 
-  df <- data_frame(
+  df <- tibble(
     id = c(1, 4),
     group = c("A", "B")
   ) %>%
@@ -672,13 +717,13 @@ test_that("grouped mutate coerces factor + character -> character (WARN) (#1892)
 })
 
 test_that("lead/lag works on more complex expressions (#1588)", {
-  df <- data_frame(x = rep(1:5, 2), g = rep(1:2, each = 5)) %>% group_by(g)
+  df <- tibble(x = rep(1:5, 2), g = rep(1:2, each = 5)) %>% group_by(g)
   res <- df %>% mutate(y = lead(x > 3))
   expect_equal(res$y, rep(lead(1:5 > 3), 2))
 })
 
 test_that("Adding a Column of NA to a Grouped Table gives expected results (#1645)", {
-  dataset <- data_frame(A = 1:10, B = 10:1, group = factor(sample(LETTERS[25:26], 10, TRUE)))
+  dataset <- tibble(A = 1:10, B = 10:1, group = factor(sample(LETTERS[25:26], 10, TRUE)))
   res <- dataset %>% group_by(group) %>% mutate(prediction = factor(NA))
   expect_true(all(is.na(res$prediction)))
   expect_is(res$prediction, "factor")
@@ -687,7 +732,7 @@ test_that("Adding a Column of NA to a Grouped Table gives expected results (#164
 
 test_that("Deep copies are performed when needed (#1463)", {
   res <- data.frame(prob = c(F, T)) %>%
-    rowwise %>%
+    rowwise() %>%
     mutate(model = list(x = prob))
   expect_equal(unlist(res$model), c(FALSE, TRUE))
 
@@ -710,40 +755,45 @@ test_that("mutate() names pronouns correctly (#2686)", {
 
 test_that("mutate() supports unquoted values", {
   df <- tibble(g = c(1, 1, 2, 2, 2), x = 1:5)
-  expect_identical(mutate(df, out = !! 1), mutate(df, out = 1))
-  expect_identical(mutate(df, out = !! 1:5), mutate(df, out = 1:5))
-  expect_identical(mutate(df, out = !! quote(1:5)), mutate(df, out = 1:5))
-  expect_error(mutate(df, out = !! 1:2), "must be length 5 (the number of rows)", fixed = TRUE)
-  expect_error(mutate(df, out = !! get_env()), "unsupported type")
+  expect_identical(mutate(df, out = !!1), mutate(df, out = 1))
+  expect_identical(mutate(df, out = !!(1:5)), mutate(df, out = 1:5))
+  expect_identical(mutate(df, out = !!quote(1:5)), mutate(df, out = 1:5))
+  expect_error(mutate(df, out = !!(1:2)), "must be length 5 (the number of rows)", fixed = TRUE)
+  expect_error(mutate(df, out = !!env(a = 1)), "unsupported type")
 
   gdf <- group_by(df, g)
-  expect_identical(mutate(gdf, out = !! 1), mutate(gdf, out = 1))
-  expect_identical(mutate(gdf, out = !! 1:5), group_by(mutate(df, out = 1:5), g))
-  expect_error(mutate(gdf, out = !! quote(1:5)), "must be length 2 (the group size)", fixed = TRUE)
-  expect_error(mutate(gdf, out = !! 1:2), "must be length 5 (the number of rows)", fixed = TRUE)
-  expect_error(mutate(gdf, out = !! get_env()), "unsupported type")
+  expect_identical(mutate(gdf, out = !!1), mutate(gdf, out = 1))
+  expect_identical(mutate(gdf, out = !!(1:5)), group_by(mutate(df, out = 1:5), g))
+  expect_error(mutate(gdf, out = !!quote(1:5)), "must be length 2 (the group size)", fixed = TRUE)
+  expect_error(mutate(gdf, out = !!(1:2)), "must be length 5 (the group size)", fixed = TRUE)
+  expect_error(mutate(gdf, out = !!env(a = 1)), "unsupported type")
 })
 
+test_that("gathering handles promotion from raw", {
+  df <- tibble(a = 1:4, g = c(1, 1, 2, 2))
+  # collecting raw in the first group, then other types
+  expect_identical(
+    df %>% group_by(g) %>% mutate(b = if (all(a < 3)) as.raw(a) else a) %>% pull(b),
+    1:4
+  )
+  expect_identical(
+    df %>% group_by(g) %>% mutate(b = if (all(a < 3)) as.raw(a) else as.numeric(a)) %>% pull(b),
+    as.numeric(1:4)
+  )
+})
 
 # Error messages ----------------------------------------------------------
 
-test_that("mutate fails gracefully on non-vector columns (#1803)", {
-  df <- data_frame(a = 1:3, b = as.raw(1:3))
-  expect_error(
-    mutate(df, a = 1),
-    "Column `b` is of unsupported type raw vector",
-    fixed = TRUE
-  )
-  expect_error(
-    mutate(df, b = 1),
-    "Column `b` is of unsupported type raw vector",
-    fixed = TRUE
-  )
-  expect_error(
-    mutate(df, c = 1),
-    "Column `b` is of unsupported type raw vector",
-    fixed = TRUE
-  )
+test_that("mutate handles raw vectors in columns (#1803)", {
+  df <- tibble(a = 1:3, b = as.raw(1:3))
+  expect_identical(mutate(df, a = 1), tibble(a = 1, b = as.raw(1:3)))
+  expect_identical(mutate(df, b = 1), tibble(a = 1:3, b = 1))
+  expect_identical(mutate(df, c = 1), tibble(a = 1:3, b = as.raw(1:3), c = 1))
+  expect_identical(mutate(df, c = as.raw(a)), tibble(a = 1:3, b = as.raw(1:3), c = as.raw(1:3)))
+
+  df <- tibble(a = 1:4, g = c(1, 1, 2, 2))
+  expect_identical(mutate(df, b = as.raw(a)) %>% group_by(g) %>% pull(b), as.raw(1:4))
+  expect_identical(mutate(df, b = as.raw(a)) %>% rowwise() %>% pull(b), as.raw(1:4))
 })
 
 test_that("grouped mutate errors on incompatible column type (#1641)", {
@@ -768,16 +818,109 @@ test_that("can reuse new variables", {
 
 test_that("can use character vectors in grouped mutate (#2971)", {
   df <-
-    data_frame(x = 1:10000) %>%
+    tibble(x = 1:10000) %>%
     group_by(x) %>%
-    mutate(y = as.character(runif(1L)),
-           z = as.character(runif(1L)))
+    mutate(
+      y = as.character(runif(1L)),
+      z = as.character(runif(1L))
+    )
 
   expect_error(df %>% distinct(x, .keep_all = TRUE), NA)
 })
 
 test_that("mutate() to UTF-8 column names", {
-  df <- data_frame(a = 1) %>% mutate("\u5e78" := a)
+  skip_on_cran()
+  df <- tibble(a = 1) %>% mutate("\u5e78" := a)
 
   expect_equal(colnames(df), c("a", "\u5e78"))
 })
+
+test_that("grouped subsets are not lazy (#3360)", {
+  make_call <- function(x) {
+    quo(!!x)
+  }
+
+  res <- tibble(name = 1:2, value = letters[1:2]) %>%
+    rowwise() %>%
+    mutate(call = list(make_call(value))) %>%
+    pull()
+
+  expect_identical(res, list(make_call("a"), make_call("b")))
+
+  res <- tibble(name = 1:2, value = letters[1:2]) %>%
+    group_by(name) %>%
+    mutate(call = list(make_call(value))) %>%
+    pull()
+
+  expect_identical(res, list(make_call("a"), make_call("b")))
+})
+
+test_that("errors don't have tracebacks (#3662)", {
+  err <- capture_condition(mutate(tibble(x = 1:10) %>% mutate(z = y)))
+  expect_null(conditionCall(err))
+
+  err <- capture_condition(n_distinct())
+  expect_null(conditionCall(err))
+})
+
+test_that("columns are no longer available when set to NULL on mutate (#3799)", {
+  tbl <- tibble(x = 1:2, y = 1:2)
+  expect_error(mutate(tbl, y = NULL, a = +sum(y)))
+  expect_error(mutate(tbl, y = NULL, a = sum(y)))
+
+  tbl <- tbl %>% group_by(x)
+  expect_error(mutate(tbl, y = NULL, a = +sum(y)))
+  expect_error(mutate(tbl, y = NULL, a = sum(y)))
+})
+
+test_that("rlang lambda inherit from the data mask (#3843)", {
+  res <- iris %>%
+    mutate_at(
+      vars(starts_with("Petal")),
+      ~ ifelse(Species == "setosa" & . < 1.5, NA, .)
+    )
+  expected <- iris %>%
+    mutate(
+      Petal.Length = ifelse(Species == "setosa" & Petal.Length < 1.5, NA, Petal.Length),
+      Petal.Width  = ifelse(Species == "setosa" & Petal.Width  < 1.5, NA, Petal.Width)
+    )
+  expect_equal(res, expected)
+
+  res <- iris %>%
+    group_by(Species) %>%
+    mutate_at(
+      vars(starts_with("Petal")),
+      ~ ifelse(Species == "setosa" & . < 1.5, NA, .)
+    )
+  expected <- iris %>%
+    group_by(Species) %>%
+    mutate(
+      Petal.Length = ifelse(Species == "setosa" & Petal.Length < 1.5, NA, Petal.Length),
+      Petal.Width  = ifelse(Species == "setosa" & Petal.Width  < 1.5, NA, Petal.Width)
+    )
+  expect_equal(res, expected)
+})
+
+test_that("mutate() does not segfault when setting an unknown column to NULL (#4035)", {
+  expect_true(all_equal(mutate(mtcars, dummy = NULL), mtcars))
+})
+
+test_that("mutate() skips evaluation of R expression for empty groups (#4088)", {
+  count <- 0
+
+  d <- tibble(f = factor(c("a", "b"), levels = c("a", "b", "c"))) %>%
+    group_by(f)
+  res <- mutate(d, x = { count <<- count + 1; 675} )
+  expect_equal(count, 2L)
+
+  d <- tibble(f = factor(c("c"), levels = c("a", "b", "c"))) %>%
+    group_by(f)
+  res <- mutate(d, x = { count <<- count + 1; 675} )
+  expect_equal(count, 3L)
+
+  res <- tibble(f = factor(levels = c("a", "b", "c"))) %>%
+    group_by(f, .drop = FALSE) %>%
+    mutate(x = { count <<- count + 1; 675} )
+  expect_equal(count, 4L)
+  expect_is(res$x, "numeric")
+})
